#include "collision.hpp"
#include "config.hpp"
#include "player.hpp"
#include "state.hpp"

void PBuildCollisionFromMap(Map *map) {
  SDL_Log("Building collision data from map");
  state.collisions.clear();

  // Tiles have no collision data attached to them, so we have to basically
  // "guess" if they are solid or not
  // Except this guess works 99% of the time
  // Look tiles up by name in the collision database

  int x = 0;
  int y = 0;
  for (std::string tileName : map->data.mapTiles) {

    if (x + 1 > map->mapWidth) {
      x = 0;
      y++;
    }

    if (state.collisionDb.tileInfo.at(tileName).isSolid) {
      PCollisionRect collisionRect{};
      collisionRect.rect.x = x * TEXTURE_WIDTH;
      collisionRect.rect.y = y * TEXTURE_HEIGHT;
      collisionRect.rect.w = TEXTURE_WIDTH - 5;
      collisionRect.rect.h = TEXTURE_HEIGHT - 5;

      state.collisions.push_back(collisionRect);
    }
    x++;
  }
}

void PPopulateCollisionDb() {
  SDL_Log("Populating collision database for the first time");
  state.collisionDb.tileInfo.insert({"WALL", {true}});
  state.collisionDb.tileInfo.insert({"FLOOR", {false}});
}

bool PCanMoveTo(int x, int y) {
  SDL_Rect playerRect = PGetPlayerRect(&state.player);
  playerRect.x = x;
  playerRect.y = y;
  for (PCollisionRect rect : state.collisions) {
    if (SDL_HasIntersection(&playerRect, &rect.rect)) {
      return false;
    }
  }

  return true;
}
